package com.fuyuanshen.equipment.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fuyuanshen.common.core.domain.PageResult;
import com.fuyuanshen.common.core.utils.PageUtil;
import com.fuyuanshen.equipment.constants.DeviceConstants;
import com.fuyuanshen.equipment.domain.Device;
import com.fuyuanshen.equipment.domain.form.DeviceForm;
import com.fuyuanshen.equipment.domain.query.DeviceQueryCriteria;
import com.fuyuanshen.equipment.domain.vo.CustomerVo;
import com.fuyuanshen.equipment.mapper.DeviceMapper;
import com.fuyuanshen.equipment.service.DeviceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.List;

/**
 * @Description:
 * @Author: WY
 * @Date: 2025/5/16
 **/
@Slf4j
@Service
@RequiredArgsConstructor
public class DeviceServiceImpl extends ServiceImpl<DeviceMapper, Device> implements DeviceService {

    @Value("${file.device.pic}")
    private String filePath;
    @Value("${file.device.ip}")
    private String ip;


    private final DeviceMapper deviceMapper;


    /**
     * 分页查询设备
     *
     * @param criteria 条件
     * @param page     分页参数
     * @return
     * @throws IOException
     */
    @Override
    public PageResult<Device> queryAll(DeviceQueryCriteria criteria, Page<Object> page) throws IOException {

        IPage<Device> devices = deviceMapper.findAll(criteria, page);

        List<Device> records = devices.getRecords();
        for (Device record : records) {
            if (record.getDevicePic() == null) {
                record.setDevicePic("");
            }
        }

        return PageUtil.toPage(devices);
    }


    /**
     * 查询所有设备
     *
     * @param criteria 筛选条件
     * @return
     */
    @Override
    public List<Device> queryAllDevices(DeviceQueryCriteria criteria) {
        return deviceMapper.findAllDevices(criteria);
    }


    /**
     * 新增设备
     *
     * @param deviceForm
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addDevice(DeviceForm deviceForm) throws Exception {

        // 获取当前登录用户信息
        // LoginUser user = LoginHelper.getLoginUser();

        // 保存图片并获取URL
        String imageUrl = saveDeviceImage(deviceForm.getFile(), deviceForm.getDeviceName());
        // 设置图片路径
        deviceForm.setDevicePic(imageUrl);

        // 转换对象并插入数据库
        Device device = new Device();
        BeanUtil.copyProperties(deviceForm, device, true);

        deviceMapper.insert(device);

    }


    /**
     * 更新设备信息
     *
     * @param deviceForm /
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(DeviceForm deviceForm) throws Exception {

        Device device = getById(deviceForm.getId());
        if (device == null) {
            throw new Exception("设备不存在!!!");
        }

        // 处理上传的图片
        String imageUrl = saveDeviceImage(deviceForm.getFile(), device.getDeviceName());
        deviceForm.setDevicePic(imageUrl);

        // 更新字段
        BeanUtil.copyProperties(deviceForm, device, true);
        device.setUpdateTime(new Timestamp(System.currentTimeMillis()));
        deviceMapper.updateById(device);
    }


    /**
     * 保存设备图片并返回访问路径
     *
     * @param file      MultipartFile
     * @param deviceMac 设备MAC用于生成唯一文件名
     * @return 文件存储路径 URL 形式
     */
    private String saveDeviceImage(MultipartFile file, String deviceMac) throws IOException {
        if (file == null || file.isEmpty()) {
            return null;
        }

        String originalFileName = file.getOriginalFilename();
        String fileExtension = originalFileName.substring(originalFileName.lastIndexOf(".") + 1);
        String newFileName = "PS_" + deviceMac + "." + fileExtension;

        File newFile = new File(filePath + DeviceConstants.FILE_ACCESS_ISOLATION + File.separator + newFileName);

        if (!newFile.getParentFile().exists()) {
            newFile.getParentFile().mkdirs();
        }

        log.info("图片保存路径: {}", newFile.getAbsolutePath());
        file.transferTo(newFile);

        return ip + DeviceConstants.FILE_ACCESS_PREFIX + "/" + DeviceConstants.FILE_ACCESS_ISOLATION + "/" + newFileName;
    }


    /**
     * 删除设备
     *
     * @param ids /
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteAll(List<Long> ids) {
        deviceMapper.deleteByIds(ids);
    }


    /**
     * 分配客户
     *
     * @param customerVo
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void assignCustomer(CustomerVo customerVo) {


    }


    /**
     * 撤回设备
     *
     * @param deviceForm
     */
    @Override
    public void withdrawDevice(DeviceForm deviceForm) {


    }


    /**
     * 解绑设备
     *
     * @param deviceForm
     */
    @Override
    @Transactional
    public void unbindDevice(DeviceForm deviceForm) {

    }




}
